
/*
 * pbrt source code Copyright(c) 1998-2005 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For educational use only; commercial use expressly forbidden.
 * NO WARRANTY, express or implied, for this software.
 * (See file License.txt for complete license)
 */

// Ice.cpp*
#include "pbrt.h"
#include "material.h"
#include "texture.h"
// Ice Class Declarations
class Ice : public Material {
public:
	// Ice Public Methods
	Ice(Reference<Texture<Spectrum> > r, Reference<Texture<Spectrum> > t,
			Reference<Texture<float> > i, Reference<Texture<float> > i1, 
			Reference<Texture<float> > bump) {
		Kr = r;
		Kt = t;
		index = i;
		index1 = i1;
		bumpMap = bump;
	}
	BSDF *GetBSDF(const DifferentialGeometry &dgGeom, const DifferentialGeometry &dgShading) const;
private:
	// Ice Private Data
	Reference<Texture<Spectrum> > Kr, Kt;
	Reference<Texture<float> > index;
	Reference<Texture<float> > index1;
	Reference<Texture<float> > bumpMap;
};
// Ice Method Definitions
BSDF *Ice::GetBSDF(const DifferentialGeometry &dgGeom, const DifferentialGeometry &dgShading) const {
	// Allocate _BSDF_, possibly doing bump-mapping with _bumpMap_
	DifferentialGeometry dgs;
	if (bumpMap)
		Bump(bumpMap, dgGeom, dgShading, &dgs);
	else
		dgs = dgShading;
	BSDF *bsdf = BSDF_ALLOC(BSDF)(dgs, dgGeom.nn);
	Spectrum R = Kr->Evaluate(dgs).Clamp();
	Spectrum T = Kt->Evaluate(dgs).Clamp();
	float ior = index->Evaluate(dgs);
	float ior1 = index1->Evaluate(dgs);
	if (!R.Black())
		bsdf->Add(BSDF_ALLOC(SpecularReflection)(R,
			BSDF_ALLOC(FresnelDielectric)(ior1, ior)));
	if (!T.Black())
		bsdf->Add(BSDF_ALLOC(SpecularTransmission)(Spectrum(1.0), ior1, ior));
	if (ior1 > 1.0)
	{
		//bsdf->Add(BSDF_ALLOC(Lambertian)(0.000001));
		MicrofacetDistribution *dist = BSDF_ALLOC(Blinn)(5.0);
		BxDF *fresnel =
			BSDF_ALLOC(FresnelBlend)(Spectrum(0.25), Spectrum(0.001), dist);
		//BSDF_ALLOC(FresnelBlend)(Spectrum(0.35), Spectrum(0.001), dist);
		bsdf->Add(fresnel);
	}
	else
	{
		MicrofacetDistribution *dist = BSDF_ALLOC(Blinn)(5.0);
		BxDF *fresnel =
			BSDF_ALLOC(FresnelBlend)(Spectrum(1.0), Spectrum(1.0), dist);
		bsdf->Add(fresnel);
	}
	return bsdf;
}
extern "C" DLLEXPORT Material * CreateMaterial(const Transform &xform,
		const TextureParams &mp) {
	Reference<Texture<Spectrum> > Kr = mp.GetSpectrumTexture("Kr", Spectrum(1.f));
	Reference<Texture<Spectrum> > Kt = mp.GetSpectrumTexture("Kt", Spectrum(1.f));
	Reference<Texture<float> > index = mp.GetFloatTexture("index", 1.31f);
	Reference<Texture<float> > index1 = mp.GetFloatTexture("indexnext", 1.0f);
	Reference<Texture<float> > bumpMap = mp.GetFloatTexture("bumpmap", 0.f);
	return new Ice(Kr, Kt, index, index1, bumpMap);
}